import { EventPlugin, EventType, PluginType, UpdateType } from '@segment/analytics-react-native';
import AsyncStorage from '@react-native-async-storage/async-storage';
import { AppState } from 'react-native';
const MAX_SESSION_TIME_IN_MS = 300000;
const SESSION_ID_KEY = 'previous_session_id';
const LAST_EVENT_TIME_KEY = 'last_event_time';
const AMP_SESSION_START_EVENT = 'session_start';
const AMP_SESSION_END_EVENT = 'session_end';
export class AmplitudeSessionPlugin extends EventPlugin {
  type = PluginType.enrichment;
  key = 'Actions Amplitude';
  active = false;
  sessionId = -1;
  lastEventTime = -1;
  configure = async analytics => {
    this.analytics = analytics;
    await this.loadSessionData();
    AppState.addEventListener('change', this.handleAppStateChange);
  };
  update(settings, type) {
    if (type !== UpdateType.initial) {
      return;
    }
    this.active = settings.integrations?.hasOwnProperty(this.key) ?? false;
  }
  async execute(event) {
    if (!this.active) {
      return event;
    }
    if (this.sessionId === -1 || this.lastEventTime === -1) {
      await this.loadSessionData();
    }
    await this.startNewSessionIfNecessary();
    let result = event;
    switch (result.type) {
      case EventType.IdentifyEvent:
        result = this.identify(result);
        break;
      case EventType.TrackEvent:
        result = this.track(result);
        break;
      case EventType.ScreenEvent:
        result = this.screen(result);
        break;
      case EventType.AliasEvent:
        result = this.alias(result);
        break;
      case EventType.GroupEvent:
        result = this.group(result);
        break;
    }
    this.lastEventTime = Date.now();
    await this.saveSessionData();
    return result;
  }
  identify(event) {
    return this.insertSession(event);
  }
  track(event) {
    return this.insertSession(event);
  }
  screen(event) {
    event.properties = {
      ...event.properties,
      name: event.name
    };
    return this.insertSession(event);
  }
  group(event) {
    return this.insertSession(event);
  }
  alias(event) {
    return this.insertSession(event);
  }
  async reset() {
    this.sessionId = -1;
    this.lastEventTime = -1;
    await AsyncStorage.removeItem(SESSION_ID_KEY);
    await AsyncStorage.removeItem(LAST_EVENT_TIME_KEY);
  }
  insertSession = event => {
    const integrations = event.integrations || {};
    const existingIntegration = integrations[this.key];
    const hasSessionId = typeof existingIntegration === 'object' && existingIntegration !== null && 'session_id' in existingIntegration;
    if (hasSessionId) {
      return event;
    }
    return {
      ...event,
      integrations: {
        ...integrations,
        [this.key]: {
          session_id: this.sessionId
        }
      }
    };
  };
  onBackground = () => {
    this.lastEventTime = Date.now();
    this.saveSessionData();
  };
  onForeground = () => {
    this.startNewSessionIfNecessary();
  };
  async startNewSessionIfNecessary() {
    const current = Date.now();
    const sessionExpired = this.sessionId === -1 || this.lastEventTime === -1 || current - this.lastEventTime >= MAX_SESSION_TIME_IN_MS;

    // Avoid loop: if session just started recently, skip restarting
    if (!sessionExpired || current - this.sessionId < 1000) {
      return;
    }
    await this.endSession();
    await this.startNewSession();
  }
  async startNewSession() {
    this.sessionId = Date.now();
    this.lastEventTime = this.sessionId;
    await this.saveSessionData();
    this.analytics?.track(AMP_SESSION_START_EVENT, {
      integrations: {
        [this.key]: {
          session_id: this.sessionId
        }
      }
    });
  }
  async endSession() {
    if (this.sessionId === -1) {
      return;
    }
    this.analytics?.track(AMP_SESSION_END_EVENT, {
      integrations: {
        [this.key]: {
          session_id: this.sessionId
        }
      }
    });
  }
  async loadSessionData() {
    const storedSessionId = await AsyncStorage.getItem(SESSION_ID_KEY);
    const storedLastEventTime = await AsyncStorage.getItem(LAST_EVENT_TIME_KEY);
    this.sessionId = storedSessionId != null ? Number(storedSessionId) : -1;
    this.lastEventTime = storedLastEventTime != null ? Number(storedLastEventTime) : -1;
  }
  async saveSessionData() {
    await AsyncStorage.setItem(SESSION_ID_KEY, this.sessionId.toString());
    await AsyncStorage.setItem(LAST_EVENT_TIME_KEY, this.lastEventTime.toString());
  }
  handleAppStateChange = nextAppState => {
    if (nextAppState === 'active') {
      this.onForeground();
    } else if (nextAppState === 'background') {
      this.onBackground();
    }
  };
}
//# sourceMappingURL=AmplitudeSessionPlugin.js.map